/* vim: set shiftwidth=4 tabstop=8 softtabstop=4: */
/* $Id: libspt.h,v 1.10 2002/12/13 17:57:35 shinra Exp $ */

#ifndef LIBSPT_H
#define LIBSPT_H

#if defined(__CYGWIN__) && !defined(LIBSPT_STATIC)
# ifdef IN_LIBSPT
#  ifdef DLL_EXPORT
#   define SPT_EXPORT __declspec(dllexport)
#  else
#   define SPT_EXPORT
#  endif
# else
#  define SPT_EXPORT __declspec(dllimport)
# endif
#endif
#ifndef SPT_EXPORT
# define SPT_EXPORT
#endif

enum {
    SPT_E_NONE = 0,
    SPT_E_INVAL = 1,
    SPT_E_NOMEM = 2,
    SPT_E_UNKNOWN = 3,
    SPT_E_AGENT_VERSION = 4,
    SPT_E_AGENT_EXEC = 5,
    SPT_E_AGENT_INST = 6,
    SPT_E_AGENT_FATAL = 7,
    SPT_E_AGENT_DEAD = 8,
    SPT_E_CHOWN_FAIL = 9,   /* chmod() or chown() to slave pty failed */
    SPT_E_UTMP_FAIL = 10,
    SPT_E_NOTTY = 11,
    SPT_E_RESOURCE = 12,
    SPT_E_CLEANUP_FAIL = 13,
    SPT_E_TTYINIT_FAIL = 14,
    SPT_E_CTTY_FAIL = 15,
};

typedef struct spt_handle_tag spt_handle;
typedef void (*spt_cleanup_proc) (void *arg);

/* 
 * SPT_E_CHOWN_FAIL and SPT_E_CLEANUP_FAIL are only for warning.
 * You may ignore them.
 *
 * After fork(), REMEMBER TO CALL spt_detach(), and close masterfd
 * and slavefd yourself.
 */
extern int SPT_EXPORT spt_open_pty(spt_handle **pphandle, int *pmasterfd,
	spt_cleanup_proc cleanup, void *cleanuparg);
extern int SPT_EXPORT spt_close_pty(spt_handle *phandle);
extern SPT_EXPORT void spt_detach_handle(spt_handle *phandle);

extern int SPT_EXPORT spt_get_slavename(
	const spt_handle *phandle, char **pname);
extern void SPT_EXPORT spt_free_obj(void *pobj);
extern int SPT_EXPORT spt_init_slavefd(const spt_handle *phandle, int fd);

extern int SPT_EXPORT spt_utmp_set_host(spt_handle *phandle, const char *host);
extern int SPT_EXPORT spt_utmp_set_pid(spt_handle *phandle, pid_t pid);
extern int SPT_EXPORT spt_utmp_set_exit(spt_handle *phandle, int exit_st);
extern int SPT_EXPORT spt_login_utmp(spt_handle *phandle);
extern int SPT_EXPORT spt_logout_utmp(spt_handle *phandle);
extern int SPT_EXPORT spt_login_wtmp(spt_handle *phandle);
extern int SPT_EXPORT spt_logout_wtmp(spt_handle *phandle);
extern int SPT_EXPORT spt_update_lastlog(spt_handle *phandle);

/* utility functions */
extern int SPT_EXPORT spt_open_slave(
	const spt_handle *phandle); /* check errno */
extern int SPT_EXPORT spt_set_ctty(int slavefd); /* check errno */

#endif /* LIBSPT_H */
