/* vim: set shiftwidth=4 tabstop=8 softtabstop=4: */
/* $Id: test.c,v 1.6 2002/12/13 18:27:16 shinra Exp $ */
#define LIBSPT_STATIC
#define USE_MEMORY_DEBUG
#ifdef linux
# define _GNU_SOURCE
#endif
#include <sys/types.h>
#include "libspt.h"
#include "config.h"
#include "cfuncs.h"
#include <stdio.h>
#include <sys/param.h>
#include <unistd.h>
#include <errno.h>
#include <signal.h>
#include <sys/ioctl.h>
#include <sys/wait.h>
#include <string.h>
#include <termios.h>
#include <sys/uio.h>

int
main(int argc, char *const argv[])
{
    int r;
    spt_handle *phandle;
    int masterfd, slavefd;

    errno = 0;
    r = spt_open_pty(&phandle, &masterfd, NULL, NULL);
    perror(NULL);
    fprintf(stderr, "r = %d\n", r);
    if (!r || r == SPT_E_CHOWN_FAIL) {
	pid_t pid;
	slavefd = spt_open_slave(phandle);
	if (slavefd < 0) {
	    perror("open slave");
	    spt_close_pty(phandle);
	    return 1;
	}
	fprintf(stderr, "slavefd = %d\n", slavefd);
	fprintf(stderr, "init_slavefd = %d\n",
		spt_init_slavefd(phandle, slavefd));
	pid = fork();
	if (pid == 0) {
	    struct termios tio;
	    struct winsize wsize;
	    spt_detach_handle(phandle);
	    setsid();
	    if (spt_set_ctty(slavefd)) {
		fputs("cannot change controlling tty\n", stderr);
		_exit(1);
	    }
	    close(masterfd);
	    tcgetattr(0, &tio);
	    cfsetispeed(&tio, cfgetispeed(&tio));
	    cfsetospeed(&tio, cfgetospeed(&tio));
	    errno = 0;
	    tcsetattr(slavefd, TCSADRAIN, &tio);
	    perror("tcsetattr");
	    ioctl(0, TIOCGWINSZ, &wsize);
	    ioctl(slavefd, TIOCSWINSZ, &wsize);
	    dup2(slavefd, 0);
	    dup2(slavefd, 1);
	    dup2(slavefd, 2);
	    close(slavefd);
	    execl("/bin/sh", "sh", NULL);
	    /* NOTREACHED */
	    perror(NULL);
	    _exit(1);
	} else {
	    struct termios old, new;
	    close(slavefd);
	    signal(SIGINT, SIG_IGN);
	    spt_utmp_set_host(phandle, "main2host");
	    spt_utmp_set_pid(phandle, pid);
	    fprintf(stderr, "login_utmp = %d\n", 
		    spt_login_utmp(phandle));
	    fprintf(stderr, "login_wtmp = %d\n", 
		    spt_login_wtmp(phandle));
	    fprintf(stderr, "update_lastlog = %d\n", 
		    spt_update_lastlog(phandle));
	    tcgetattr(0, &old);
#if 0
	    cfmakeraw(&new);
#else
	    new = old;
	    new.c_iflag &= ~(IXOFF|INPCK|BRKINT|PARMRK|ISTRIP|INLCR|IGNCR|ICRNL|IXON|IGNPAR);
#ifdef IMAXBEL
	    new.c_iflag &= ~IMAXBEL;
#endif
	    new.c_iflag |= IGNBRK;
	    new.c_oflag = ~OPOST;
	    new.c_lflag &= ~(ECHO|ECHOE|ECHOK|ECHONL|ICANON|ISIG|IEXTEN|NOFLSH|TOSTOP);
#ifdef PENDIN
	    new.c_lflag &= PENDIN;
#endif
	    new.c_cflag &= ~(CSIZE|PARENB);
	    new.c_cflag |= CS8|CREAD;
	    new.c_cc[VMIN] = 1;
	    new.c_cc[VTIME] = 0;
#endif
	    tcsetattr(0, TCSADRAIN, &new);
	    tcsetattr(1, TCSADRAIN, &new);
	    while (1) {
		ssize_t r;
		int rr;
		char buf[30];
		fd_set fds;
		FD_ZERO(&fds);
		FD_SET(masterfd, &fds);
		FD_SET(0, &fds);
		rr = select(masterfd + 1, &fds, NULL, NULL, NULL);
		if (rr <= 0)
		    break;
		if (FD_ISSET(masterfd, &fds)) {
		    r = read(masterfd, buf, sizeof buf);
		    if (r <= 0)
			break;
		    write(1, buf, r);
		}
		if (FD_ISSET(0, &fds)) {
		    r = read(0, buf, sizeof buf);
		    if (r <= 0)
			break;
		    if (buf[0] == '!')
			break;
		    write(masterfd, buf, r);
		}
	    }
	    tcsetattr(0, TCSADRAIN, &old);
	    tcsetattr(1, TCSADRAIN, &old);
	    fprintf(stderr, "logout_utmp = %d\n", 
		    spt_logout_utmp(phandle));
	    fprintf(stderr, "logout_wtmp = %d\n", 
		    spt_logout_wtmp(phandle));
	    spt_close_pty(phandle);
	    kill(pid, SIGTERM);
	    /*
	    if (waitpid(pid, NULL, WNOHANG) != pid) {
		sleep(1);
		if (waitpid(pid, NULL, WNOHANG) != pid)
		    fprintf(stderr, "warning: wait failed\n");
	    }
	    */
	    MEMORY_REPORT();
	    return 0;
	}
    }
    return r;
}
